/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki;

import java.util.Calendar;
import java.util.Date;

import org.jboss.logging.Logger;
import org.jboss.wiki.exceptions.PageNotEditedException;

public class EditSessionWatcher extends Thread {
	private Integer editSessionId;

	private Date endTime;

	private WikiPage watchedPage;

	private boolean runs = true;
	
	private Logger log;

	public EditSessionWatcher(Integer editSessionId, int sessionIdExpireTime,
			WikiPage watchedPage) {
		
		log = Logger.getLogger(EditSessionWatcher.class);
		
		this.editSessionId = editSessionId;
		this.watchedPage = watchedPage;
		Calendar cal = Calendar.getInstance();

		cal.setTime(new Date());
		cal.add(Calendar.MINUTE, sessionIdExpireTime);
		cal.add(Calendar.SECOND, -1);	//so it doesn't wait one minute longer
		
		endTime = cal.getTime();
	}

	@Override
	public void run() {
		while (runs) {
			log.debug("Checking watcher for page "+watchedPage.getName()+" with sessionid: "+editSessionId);
			Date now = new Date();

			if (now.after(endTime)) {
				log.debug("Session time exceeded for page "+watchedPage.getName()+" with sessionid: "+editSessionId);
				
				try {
					watchedPage.addExpiredSessionId(editSessionId);
				} catch (PageNotEditedException e) {
					e.printStackTrace();
				}
				stopWatcher();
			}
			else {
				try {
					sleep(60000);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
		}
	}

	public void stopWatcher() {
		log.debug("Stopping watcher for page: "+watchedPage.getName()+" with sessionid: "+editSessionId);
		runs = false;
	}

}
