/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Date;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;
import java.util.TreeSet;

import org.jboss.logging.Logger;
import org.jboss.wiki.exceptions.PageRenamingException;

/**
 * <p>
 * </p>
 * 
 */
public class FileDataSource implements MediaDataSource, AttachmentDataSource,
		WikiPageDictionary {

	/**
	 * <p>
	 * Represents ...
	 * </p>
	 * 
	 */
	private final String SAVE_COMMENT = "Saved by FileDataSource of JBoss Wiki";

	public static final String propFileName = "filedatasource.properties";

	public static final String propModFileName = "fileMod.properties";

	public String pathToMediaTrash;

	public String pathToAttTrash;

	private String pathToMedia;

	private String pathToAttachments;

	private boolean lock = false;

	// private Properties fileDSProps;

	private final int BUF_SIZE = 32768;

	private Properties pageModProps;

	private Properties pageDictionary;

	private Properties pageRevDictionary;

	private File modFile;

	private File pageDictionaryFile;

	private WikiEngine wikiEngine;
	
	private Logger log;

	public void setWikiEngine(WikiEngine wikiEngine) {
		this.wikiEngine = wikiEngine;
	}

	public FileDataSource() {
		log = Logger.getLogger(FileDataSource.class);
		
		log.info("Looking for props file !");

		Properties fileDSProps = new Properties();

		try {

			fileDSProps.load(FileDataSource.class
					.getResourceAsStream(propFileName));

		} catch (IOException ioe) {
			log.error("Can't load the file " + propFileName + "\n"
					, ioe);
		}

		loadProperties(fileDSProps);

		File mainWikiDir = new File(pathToMedia);
		File attWikiDir = new File(pathToAttachments);
		
		if (!mainWikiDir.exists())
			mainWikiDir.mkdirs();
		
		if (!attWikiDir.exists())
			attWikiDir.mkdirs();
		
		pageModProps = new Properties();
		modFile = new File(pathToMedia + "/" + propModFileName);

		FileInputStream fis = null;

		try {
			if (!modFile.exists()) {
				modFile.createNewFile();
			}

			fis = new FileInputStream(modFile);

			pageModProps.load(fis);

		} catch (IOException ioe) {
			log.error("Can't load the file " + propModFileName + "\n"
					, ioe);
		} finally {
			try {
				fis.close();
			} catch (IOException ioe) {
				log.error("Can't close input stream \n" , ioe);
			}
		}

		// create trash directory for deleted pages if it doesn't exists

		pathToMediaTrash = pathToMedia + "/" + MediaDataSource.mediaTrashName;

		File pathToTrashFile = new File(pathToMediaTrash + "/OLD");

		if (!pathToTrashFile.exists()) {
			pathToTrashFile.mkdirs();
		}

		// create trash directory for deleted attachments if it doesn't exists

		pathToAttTrash = pathToMedia + "/" + AttachmentDataSource.attTrashName;

		File pathToAttachmentsFile = new File(pathToAttTrash);

		if (!pathToAttachmentsFile.exists()) {
			pathToAttachmentsFile.mkdirs();
		}

		// load dictionary props
		pageDictionary = new Properties();
		pageRevDictionary = new Properties();

		// TODO Do not load dictionary YET !

		pageDictionaryFile = new File(pathToMedia + "/dictionary.properties");

		if (!pageDictionaryFile.exists()) {
			try {
				pageDictionaryFile.createNewFile();
			} catch (IOException e) {
				log.error("Cannot create dictionary file: ",e);
			}
		} else {
			// there is dictionary file. Load props from it
			FileInputStream is;
			try {
				is = new FileInputStream(pageDictionaryFile);

				try {
					pageDictionary.load(is);
				} catch (IOException e) {
					e.printStackTrace();
				}

				try {
					is.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			}

		}

		// create pageRevDictionary - to make it work faster

		Set<Object> keys = pageDictionary.keySet();

		for (Object key : keys) {
			pageRevDictionary.setProperty(pageDictionary
					.getProperty((String) key), (String) key);
		}
	}

	public void loadProperties(Properties fileDSProps) {
		if (fileDSProps.getProperty("pathToMedia") != null) {
			pathToMedia = fileDSProps.getProperty("pathToMedia");
			log.info("Path to media is: " + pathToMedia);
			pathToAttachments = fileDSProps.getProperty("pathToAttachements");
			log.info("Path to attachements is: " + pathToAttachments);
		} else {
			System.err
					.println("Your properties file lacks of pathToMedia property. It will not run properly ! Please update "
							+ propFileName);
		}
	}

	public boolean preSave() {
		return true;
	}

	public boolean savePage(String uid, WikiPage page, String languageCode) {
		// TODO Languages

		boolean status = false;

		if (pageExists(uid)) { // page exists on
			// harddisk - add new
			// version
			Properties pageProps = getPageProps(uid);

			pageProps.setProperty(page.getLastVersion() + ".author", page
					.getLastAuthor().getName());

			try {
				pageProps.store(new FileOutputStream(new File(pathToMedia
						+ "/OLD/" + uid + "/page.properties")),
						"Saved by FileDataSource of JBoss Wiki");
			} catch (Exception e) {
				System.err
						.println("[FILEDATASOURCE]: Cannot save property file: "
								+ e.toString());
			}

			log.info("Opening files");
			File pageFile = new File(pathToMedia + "/OLD/" + uid
					+ "/" + (page.getLastVersion() - 1) + ".txt");
			File pageOldFile = new File(pathToMedia + "/" + uid
					+ ".txt");

			FileOutputStream pageFileWriter;
			FileInputStream pageFileReader;

			try {
				pageFileWriter = new FileOutputStream(pageFile);
				pageFileReader = new FileInputStream(pageOldFile);

				int b;

				while ((b = pageFileReader.read()) != -1) {
					pageFileWriter.write(b);
				}

				pageFile.setLastModified(pageOldFile.lastModified());

				pageFileWriter.close();

				pageOldFile.delete();

				pageFileWriter = new FileOutputStream(pageOldFile);

				String pageContent = page.getContent();

				for (int i = 0; i < pageContent.length(); i++) {
					pageFileWriter.write((int) pageContent.charAt(i));
				}

				pageFileWriter.close();
				status = true;
			} catch (FileNotFoundException fnfe) {
				log.error("[FILEDATASOURCE]: Cannot save page: "
						, fnfe);
			} catch (IOException ioe) {
				log.error("[FILEDATASOURCE]: Cannot save page: "
						, ioe);
			}

		} else { // there is no page on disk. We have to create new one.

			try {

				// create needed dirs
				File newPageFile = new File(pathToMedia + "/OLD/"
						+ uid);
				newPageFile.mkdirs();

				// write the page
				newPageFile = new File(pathToMedia + "/OLD/" + uid
						+ "/page.properties");

				FileOutputStream pageWriter = new FileOutputStream(newPageFile);

				Properties properties = new Properties();
				properties.setProperty(page.getLastVersion() + ".author", page
						.getLastAuthor().getName());

				properties.store(pageWriter, SAVE_COMMENT);

				pageWriter.close();

				String pageDirName = pathToMedia;
				String[] pageNames = uid.split("/");

				for (int i = 0; i < pageNames.length - 1; i++) {
					pageDirName += "/" + pageNames[i];
				}

				File pageDir = new File(pageDirName);

				pageDir.mkdirs();

				pageWriter = new FileOutputStream(new File(pathToMedia + "/"
						+ uid + ".txt"));

				String pageContent = page.getContent();

				for (int i = 0; i < pageContent.length(); i++) {
					pageWriter.write((int) pageContent.charAt(i));
				}

				pageWriter.close();

				status = true;
			} catch (Exception e) {
				log.error("Cannot write new page: ", e);
				status = false;
			}
		}
		return status;
	}

	public boolean postSave() {
		return true;
	}

	public boolean preGet() {
		return true;
	}

	public Properties getPageProps(String pageName) {
		Properties pageProps = new Properties();

		try {
			FileInputStream fpi = new FileInputStream(new File(pathToMedia
					+ "/OLD/" + pageName + "/page.properties"));
			pageProps.load(fpi);
			fpi.close();
		} catch (IOException ioe) {
			// There is no prop file for some reson - create one.

			File pagePropsFile = new File(pathToMedia + "/OLD/" + pageName);

			// create dirs
			pagePropsFile.mkdirs();

			// create file
			pagePropsFile = new File(pathToMedia + "/OLD/" + pageName
					+ "/page.properties");

			try {
				pagePropsFile.createNewFile();

				if (pageExists(pageName)) {
					// if page exists it looks like there is lack of the props
					// structure
					FileOutputStream fpo = new FileOutputStream(pagePropsFile);

					pageProps.setProperty("1.author", "unknown");
					pageProps.store(fpo, SAVE_COMMENT);
					fpo.close();
				}

			} catch (IOException e) {
				log.error("Cannot create page props file for page: "
						+ pageName,e);
			}
			return pageProps;
		}

		return pageProps;
	}

	public Properties getAttProps(String pageName, String attName,
			boolean createProps) {
		Properties attProps = new Properties();

		File propFile = new File(pathToAttachments + "/" + pageName + "-att/"
				+ attName + "-dir/");

		if (!propFile.exists()) {

			if (createProps) {
				// create dirs for this property file
				propFile.mkdirs();
			} else {
				return null;
			}
		}

		propFile = new File(pathToAttachments + "/" + pageName + "-att/"
				+ attName + "-dir/attachment.properties");

		try {
			attProps.load(new FileInputStream(propFile));
		} catch (IOException ioe) {
			System.err
					.println("Cannot read attachment.properties for attachement: "
							+ pathToAttachments
							+ "/"
							+ pageName
							+ "-att/"
							+ attName + "-dir/attachment.properties\n" + ioe);
		}

		return attProps;
	}

	public void saveAttProps(Properties props, String pageName, String attName) {
		try {
			props.store(new FileOutputStream(new File(pathToAttachments + "/"
					+ pageName + "-att/" + attName
					+ "-dir/attachment.properties")), SAVE_COMMENT);
		} catch (Exception ioe) {
			System.err
					.println("Cannot store attchement.properties for attachement: "
							+ pathToAttachments
							+ "/"
							+ pageName
							+ "-att/"
							+ attName + "-dir/attachments.properties\n" + ioe);
		}

	}

	public WikiPage getPage(String pageName) {
		// log.info("[WIKI:FileDataSource]: Getting new page: " +
		// pageName);

		WikiPage page = null;

		File pageFile = new File(pathToMedia + "/" + pageName + ".txt");

		if (pageFile.exists()) {

			Properties pageProps = getPageProps(pageName);

			if (pageProps != null) {

				int i = 1;

				// get the author of the latest version of page
				while (pageProps.getProperty(String.valueOf(i + 1) + ".author") != null) {
					i++;
				}

				Credentials author = new SimpleCredentials(pageProps
						.getProperty(String.valueOf(i) + ".author"));

				StringBuilder pageContent = new StringBuilder();

				try {
					FileInputStream input = new FileInputStream(pageFile);

					int nextChr = input.read();

					while (nextChr != -1) {
						pageContent.append((char) nextChr);
						nextChr = input.read();
					}
				} catch (IOException ioe) {
					log.error(ioe);
				}

				int pageMods = getPageMod(pageName);

				page = new WikiPage(pageName, author, pageContent.toString(),
						i, i, new Date(pageFile.lastModified()), wikiEngine,
						(pageMods & VIEWABLE) == VIEWABLE,
						(pageMods & EDITABLE) == EDITABLE);

				page.setLength(pageFile.length());
			}
		}

		return page;

	}

	public boolean postGet() {
		return true;
	}

	public WikiPage getPage(String pageName, String languageCode) {
		if (languageCode != null) {
			log.info("As for now, languages aren't supported in the wiki. Please do not use language codes");
		}
		return getPage(pageName);
	}

	public WikiPage getPageAtVersion(WikiPage originPage, boolean loadContent,
			String languageCode, int version) {
		if (languageCode != null) {
			log.info("As for now, languages aren't supported in the wiki. Please do not use language codes");
		}

		return getPageAtVersion(originPage, loadContent, version);
	}

	public WikiPage getPageAtVersion(WikiPage originPage, boolean loadContent,
			int version) {

		/*
		 * Calendar cal = Calendar.getInstance(); cal.setTime(new Date()); long
		 * oldMilis = cal.getTimeInMillis();
		 */

		WikiPage page = null;

		try {
			page = (WikiPage) originPage.clone();
		} catch (CloneNotSupportedException e) {
			log.error(e);
		}

		if (page != null) {
			if (page.getLastVersion() < version) {
				log.info("There is no version " + version
						+ " of page " + page.getName());
				return null;
			}

			getContentAtVersion(page, loadContent, version);

			page.setLastAuthor(getAuthorAtVersion(page.getName(), version));

			page.setVersion(version);

			page.setEditDate(getDateAtVersion(page.getName(), version));

		}

		/*
		 * cal.setTime(new Date());
		 * 
		 * log.info("PAGE "+page.getName()+" loaded in:
		 * "+(cal.getTimeInMillis() - oldMilis));
		 */
		return page;
	}

	public void getContentAtVersion(WikiPage page, boolean loadContent,
			int version) {
		StringBuilder ret = null;

		File pageFile = new File(pathToMedia + "/OLD/" + page.getName() + "/"
				+ version + ".txt");

		if (loadContent) {
			try {
				FileInputStream input = new FileInputStream(pageFile);

				int nextChr = input.read();

				ret = new StringBuilder();

				while (nextChr != -1) {
					ret.append((char) nextChr);
					nextChr = input.read();
				}

				page.setPageContent(ret.toString());
				page.setLength(ret.length());

			} catch (IOException ioe) {
				log.error(ioe);
			}
		} else {
			page.setLength(pageFile.length());
			page.setPageContent(null);
		}

	}

	public Date getDateAtVersion(String pageName, int version) {
		Date ret = null;

		File pageFile = new File(pathToMedia + "/OLD/" + pageName + "/"
				+ version + ".txt");

		ret = new Date(pageFile.lastModified());

		return ret;
	}

	public Credentials getAuthorAtVersion(String pageName, int version) {
		return new SimpleCredentials(getPageProps(pageName).getProperty(
				version + ".author"));
	}

	public boolean pageExists(String pageName) {

		File pageFile = new File(pathToMedia + "/" + pageName + ".txt");

		if (pageFile.exists()) {
			return true;
		} else {
			return false;
		}
	}

	public void addAttachment(File attFile, String attName, WikiPage page,
			String user) {
		String pageName = page.getName();
		Properties attProps = getAttProps(pageName, attName, true);

		int lastVersion = 1;

		if (attProps.size() > 0) {
			// find last version of this file
			while (attProps.getProperty(lastVersion + ".author") != null) {
				lastVersion++;
			}
		}

		String[] extension = attName.split("\\.");
		String fileName = pathToAttachments
				+ "/"
				+ pageName
				+ "-att/"
				+ attName
				+ "-dir/"
				+ lastVersion
				+ "."
				+ ((attName.indexOf('.') != -1) ? extension[extension.length - 1]
						: "bin");

		FileInputStream oldFile = null;
		FileOutputStream newFile = null;

		try {
			newFile = new FileOutputStream(new File(fileName));
			oldFile = new FileInputStream(attFile);
		} catch (FileNotFoundException e) {
			log.error(e);
		}

		// writing old file to new one
		try {
			byte[] buffer = new byte[BUF_SIZE];
			int read;
			while ((read = oldFile.read(buffer)) != -1)
				newFile.write(buffer, 0, read);
		} catch (Exception e2) {
			// Nothing that we can really do. Just send an incomplete
			// file.
		} finally {
			try {
				newFile.close();
				oldFile.close();
			} catch (IOException e) {
				log.error(e);
			}
		}

		attProps.setProperty(lastVersion + ".author", user);

		saveAttProps(attProps, pageName, attName);
	}

	public Set<String> getAttachmentsSet(WikiPage page) {
		Set<String> attSet = new HashSet<String>();

		File attDir = new File(pathToAttachments + "/" + page.getName()
				+ "-att");

		if (attDir.exists() && (attDir.listFiles().length > 0)) {
			File[] attachements = attDir.listFiles();

			for (int i = 0; i < attachements.length; i++) {
				if ((attachements[i].isDirectory())
						&& (attachements[i].getName().endsWith("-dir"))) {
					String[] tokens = attachements[i].getName().split("/");
					attSet.add(tokens[tokens.length - 1].substring(0,
							tokens[tokens.length - 1].length() - 4));
				}
			}
		} else {
			// page has no attachements
			return null;
		}

		return attSet;
	}

	public Set<String> getAllPageNames() {
		Set<String> attSet = new HashSet<String>();

		getPageNamesFor("", attSet);

		return attSet;
	}

	private void getPageNamesFor(String directory, Set<String> attSet) {
		File attDir = new File(pathToMedia + "/" + directory);

		File[] pageFiles = attDir.listFiles();

		for (int i = 0; i < pageFiles.length; i++) {
			if ((pageFiles[i].isFile())
					&& (pageFiles[i].getName().endsWith(".txt"))) {
				// String[] tokens = pageFiles[i].getName().split("/");
				attSet.add((directory + "/" + pageFiles[i].getName().substring(
						0, pageFiles[i].getName().length() - ".txt".length()))
						.substring(1));
			}
		}

		for (int i = 0; i < pageFiles.length; i++) {
			if ((pageFiles[i].isDirectory())
					&& (!pageFiles[i].getName().endsWith("-att"))) {

				// do not take dirs from OLD, .svn and trash folders
				if ((!pageFiles[i].getAbsolutePath().equals(
						pathToMedia + "/OLD"))
						&& (!pageFiles[i].getAbsolutePath().equals(
								pathToMedia + "/.svn"))
						&& (!pageFiles[i].getAbsolutePath().equals(
								pathToMedia + "/"
										+ MediaDataSource.mediaTrashName))
						&& (!pageFiles[i].getAbsolutePath().equals(
								pathToMedia + "/"
										+ AttachmentDataSource.attTrashName))) {
					// get names for subpages
					getPageNamesFor(directory + "/" + pageFiles[i].getName(),
							attSet);
				}
			}
		}
	}

	public WikiAttachment getAttachment(String pageName, String attachementName) {
		return getAttachment(pageName, attachementName,
				getLastAttachmentVersion(pageName, attachementName));
	}

	public WikiAttachment getAttachment(String pageName,
			String attachementName, int version) {

		String[] tokens = attachementName.split("\\.");

		Properties props = getAttProps(pageName, attachementName, false);

		if (props == null) {
			return null;
		}

		String user = props.getProperty(version + ".author");

		File file = new File(
				pathToAttachments
						+ "/"
						+ pageName
						+ "-att/"
						+ attachementName
						+ "-dir/"
						+ version
						+ "."
						+ ((attachementName.indexOf(".") != -1) ? tokens[tokens.length - 1]
								: "bin"));

		return new WikiAttachment(attachementName,
				new Date(file.lastModified()), user, file.length(), file,
				version);
	}

	public int getLastAttachmentVersion(String pageName, String attachementName) {

		Properties props = getAttProps(pageName, attachementName, false);

		if (props == null)
			return -1;

		// looking for the newest version
		int version = 1;

		while (props.getProperty((version + 1) + ".author") != null) {
			version++;
		}

		return version;
	}

	public long getAttachmentSize(String pageName, String attachementName,
			int version) {
		String[] tokens = attachementName.split("\\.");

		File file = new File(
				pathToAttachments
						+ "/"
						+ pageName
						+ "-att/"
						+ attachementName
						+ "-dir/"
						+ version
						+ "."
						+ ((attachementName.indexOf(".") != -1) ? tokens[tokens.length - 1]
								: "bin"));

		return file.length();
	}

	public Set<String> getPagesFor(String pageName) {
		Set<String> pages = new TreeSet<String>();

		File pageDir = new File(pathToAttachments + "/" + pageName);

		if ((pageDir.exists()) && (pageDir.isDirectory())) {
			File[] subFiles = pageDir.listFiles();

			for (int i = 0; i < subFiles.length; i++) {
				if ((subFiles[i].isFile())
						&& (subFiles[i].getName().endsWith(".txt"))) {
					pages.add(pageName
							+ "/"
							+ subFiles[i].getName().substring(
									0,
									subFiles[i].getName().length()
											- ".txt".length()));
				}
			}
		}

		return pages;
	}

	public int getPageMod(String pageName) {
		if (pageModProps.getProperty(pageName) == null) {
			return (EDITABLE + VIEWABLE);
		} else {
			return Integer.valueOf(pageModProps.getProperty(pageName));
		}
	}

	public synchronized void setPageMod(String pageName, int mods) {

		if (mods != (EDITABLE + VIEWABLE)) {
			pageModProps.setProperty(pageName, String.valueOf(mods));
		} else {
			pageModProps.remove(pageName);
		}

		FileOutputStream fos = null;

		try {
			fos = new FileOutputStream(modFile);
			pageModProps.store(fos, SAVE_COMMENT);
		} catch (IOException ioe) {
			log.error("Couldn't store mod props: ",ioe);
		} finally {
			try {
				fos.close();
			} catch (IOException ioe) {
				log.error("Can't close input stream \n",ioe);
			}
		}
	}

	public synchronized boolean deletePage(String pageName) {

		if (pageExists(pageName)) {
			// moving page file
			String pageFileName = pathToMedia + "/" + pageName + ".txt";
			String pageFileTrashNameBase = pathToMediaTrash + "/" + pageName
					+ ".txt";
			String pageFileTrashName = pageFileTrashNameBase;

			File pageFile = new File(pageFileName);
			File pageTrashFile = new File(pageFileTrashName);

			// create dirs to trash
			File trashFileDir = new File(pathToMediaTrash + "/" + pageName);
			if (!trashFileDir.exists()) {
				trashFileDir.mkdirs();
			}

			// look if page is already in trash (add i-bak)
			int i = 0;

			if (pageTrashFile.exists()) {
				while (pageTrashFile.exists()) {
					pageFileTrashName = pageFileTrashNameBase + "-" + (i++)
							+ "-bak";
					pageTrashFile = new File(pageFileTrashName);
				}

			}

			if (!pageFile.renameTo(pageTrashFile)) {
				log.error("[FileDataSource]: Problems with moving "
						+ pageFileName + " to " + pageFileTrashName);
				return false;
			}

			// moving history directory

			String historyDirName = pathToMedia + "/OLD/" + pageName;
			String historyTrashDirNameBase = pathToMediaTrash + "/OLD/"
					+ pageName;
			String historyTrashDirName = historyTrashDirNameBase;

			// create dirs to trash (history)
			trashFileDir = new File(pathToMediaTrash + "/OLD/" + pageName);
			if (!trashFileDir.exists()) {
				trashFileDir.mkdirs();
			}

			pageFile = new File(historyDirName);
			pageTrashFile = new File(historyTrashDirName);

			// look if history dir is already in trash (add i-bak)
			i = 0;

			if (pageTrashFile.exists()) {
				while (pageTrashFile.exists()) {
					historyTrashDirName = historyTrashDirNameBase + "-" + (i++)
							+ "-bak";
					pageTrashFile = new File(historyTrashDirName);
				}

			}

			if (!pageFile.renameTo(pageTrashFile)) {
				log.error("[FileDataSource]: Problems with moving "
						+ historyDirName + " to " + historyTrashDirName);

				// return back the page history
				pageFile = new File(pageFileName);
				pageTrashFile = new File(pageFileTrashName);

				if (!pageTrashFile.renameTo(pageFile)) {
					System.err
							.println("[FileDataSource]: ERROR. I've tried moving previously moved page file from the trash, but there was some problem. This is not good and there is nothing i can do about it. The error ocured while moving "
									+ pageFileTrashName + " to " + pageFileName);
				}

				return false;
			}

			return true;
		} else {
			// no such page
			log.error("There is now such page: " + pageName);
			return false;
		}
	}

	public synchronized boolean deleteAttachment(String pageName,
			String attachmentName) {

		String attName = pathToAttachments + "/" + pageName + "-att/"
				+ attachmentName + "-dir";
		String attTrashNameBase = pathToAttTrash + "/" + pageName + "-att/"
				+ attachmentName + "-dir";
		String attTrashName = attTrashNameBase;

		// create dirs to trash
		File trashFileDir = new File(pathToAttTrash + "/" + pageName + "-att/");
		if (!trashFileDir.exists()) {
			trashFileDir.mkdirs();
		}

		// moving file directory
		File attFile = new File(attName);

		if (attFile.exists()) {

			// look if att dir is already in trash (add i-bak)
			File attTrashFile = new File(attTrashName);

			int i = 0;

			if (attTrashFile.exists()) {
				while (attTrashFile.exists()) {
					attTrashName = attTrashNameBase + "-" + (i++) + "-bak";
					attTrashFile = new File(attTrashName);
				}

			}

			if (!attFile.renameTo(attTrashFile)) {
				log.error("[ATTACHMENT DS]:Problems with moving "
						+ attName + " to " + attTrashName);
				return false;
			}

			return true;
		} else {
			log.error("Page " + pageName + " has no " + attachmentName
					+ " attachmnet");
			return false;
		}
	}

	public synchronized boolean deleteAttachments(String pageName) {
		String attName = pathToAttachments + "/" + pageName + "-att";
		String attTrashNameBase = pathToAttTrash + "/" + pageName + "-att";
		String attTrashName = attTrashNameBase;

		// create dirs to trash
		File trashFileDir = new File(pathToAttTrash + "/" + pageName);
		if (!trashFileDir.exists()) {
			trashFileDir.mkdirs();
		}

		// moving file directory
		File attFile = new File(attName);

		if (attFile.exists()) {
			// look if att dir is already in trash (add i-bak)
			File attTrashFile = new File(attTrashName);

			int i = 0;

			if (attTrashFile.exists()) {
				while (attTrashFile.exists()) {
					attTrashName = attTrashNameBase + "-" + (i++) + "-bak";
					attTrashFile = new File(attTrashName);
				}

			}

			if (!attFile.renameTo(new File(attTrashName))) {
				log.error("[ATTACHMENT DS]:Problems with moving "
						+ attName + " to " + attTrashName);
				return false;
			}

			return true;
		} else {
			log.error("Page " + pageName + " has no attachmnets");
			return false;
		}
	}

	public String getRealName(String uid) {
		while (lock)
			; // wait till unlocked (dictionary is changing)

		return (pageDictionary.getProperty(uid) != null) ? pageDictionary
				.getProperty(uid) : uid;
	}

	public String getUid(String realName) {
		while (lock)
			; // wait till unlocked (dictionary is changing)

		return (pageRevDictionary.getProperty(realName) != null) ? pageRevDictionary
				.getProperty(realName)
				: realName;
	}

	public synchronized void rename(String uid, String newName)
			throws PageRenamingException {
		
		if (wikiEngine.pageExists(newName)) {
			throw new PageRenamingException(
					"Page with the name you're trying rename to already exists.");
		}
		
		lock = true;
		
		pageDictionary.setProperty(uid, newName);
		pageRevDictionary.setProperty(newName, uid);

		// add also maping for future name
		// find apropriate uid

		int i = 2;
		
		while ((pageExists(uid + String.valueOf(i)))
				|| (pageDictionary.containsKey(uid + String.valueOf(i)))) {
			log.info(i++);
		}

		// add maping for new (to be done) with name translated to old uid
		pageDictionary.setProperty(uid + String.valueOf(i), uid);
		pageRevDictionary.setProperty(uid, uid + String.valueOf(i));
		
		saveDictionary();

		log.info("Renamed (uid) "+uid+" to "+newName);
		
		lock = false;
	}

	private void saveDictionary() {
		// synchronize dictionary with file on disk
		FileOutputStream fos;
		try {
			fos = new FileOutputStream(pageDictionaryFile);

			try {
				pageDictionary.store(fos, SAVE_COMMENT);
			} catch (IOException e) {
				e.printStackTrace();
			}

			try {
				fos.close();
			} catch (IOException e) {
				e.printStackTrace();
			}

		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
	}

	public boolean uidInDictionary(String uid) {
		return pageDictionary.contains(uid);
	}

	public boolean realNameInDictionary(String realName) {
		return pageRevDictionary.contains(realName);
	}

	public synchronized void removeDictForPage(String uid) throws PageRenamingException {
		if (!pageDictionary.contains(uid)) {
			throw new PageRenamingException(
					"There is no dictionary entry for uid: " + uid);
		}

		String realName = pageDictionary.getProperty(uid);

		// remove all dict entries
		pageDictionary.remove(uid);
		pageRevDictionary.remove(realName);
		
		saveDictionary();
	}
}
