/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki;

import java.util.HashMap;
import java.util.Vector;

import org.jboss.wiki.exceptions.PageRenamingException;

public class RenamePageWatcher extends Thread {
	private WikiEngine wikiEngine;
	private boolean runs = false;
	private Vector<String> uids;
	private Vector<String> realNames;
	private WikiPageDictionary wikiPageDictionary;
	
	public RenamePageWatcher (WikiEngine wikiEngine, WikiPageDictionary wikiPageDictionary) {
		this.wikiEngine = wikiEngine;
		this.wikiPageDictionary = wikiPageDictionary;
		uids = new Vector<String>();
		realNames = new Vector<String>();
	}
	
	public void run () {
		runs = true;
		
		while (runs) {
			if (wikiEngine.arePagesEdited()) {
				try {
					sleep(60000);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			else {
				try {
					wikiEngine.getS().acquire();
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
				
				for (int i = 0; i < uids.size(); i++) {
					try {
						wikiPageDictionary.rename(uids.get(i), realNames.get(i));
					} catch (PageRenamingException e) {
						e.printStackTrace();
					}
				}
				
				//clear tables
				uids.clear(); realNames.clear();
				
				wikiEngine.getS().release();
				
				runs = false;
			}
		}
	}
	
	public boolean isRuning() {
		return runs;
	}
	
	public void rename(String uid, String realName) throws PageRenamingException{
		if (uids.contains(uid)) {
			throw new PageRenamingException("This page is set to be renamed.");
		}
		else if (realNames.contains(realName)){
			throw new PageRenamingException("Other page is going to be renamed to the same name.");
		}
		
		uids.add(uid);
		realNames.add(realName);
	}
	
	public void stopWatcher() {
		runs = false;
	}
	
	public HashMap<String, String> getPagesWaiting() {
		HashMap<String, String> pagesWaiting = new HashMap<String,String>();
		
		for (int i = 0; i < uids.size(); i++) {
			pagesWaiting.put(uids.get(i), realNames.get(i));
		}
		
		return pagesWaiting;
	}
}
