/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki;

import java.io.IOException;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.InvalidPropertiesFormatException;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.TreeSet;
import java.util.Map.Entry;
import java.util.concurrent.Semaphore;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.portlet.ActionRequest;
import javax.portlet.RenderRequest;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.jboss.wiki.exceptions.PageNotEditedException;
import org.jboss.wiki.exceptions.PageRenamingException;
import org.jboss.wiki.exceptions.WikiException;

import org.jboss.logging.Logger;

import java.io.File;

/**
 * 
 * @author <a href="mailto:dsicore@jboss.org">Damon Sicore</a>
 * @author <a href="mailto:tomasz.szymanski@jboss.com">Tomasz Szymanski</a>
 * @version $Id$
 * 
 * getLink - gets a link to a given page getEditLink - same but link to edit
 * that page save - saves page with given attributes getContent - retrieves the
 * content from a wiki page and returns it in the type specified by 'type'
 * Probably need to have some sort of WikiSession per request with an associated
 * WikiContext, but maybe not--don't want to duplicate PortletSession
 */
public class WikiEngine {

	/**
	 * <p>
	 * </p>
	 * 
	 */
	public static String defaultPage = "Main";

	public final String propertiesFileName = "JBossWiki.properties";

	private HashMap<String, WikiPage> pages;

	private HashMap<String, WikiInsidePlugin> insidePlugins;

	private HashMap<String, WikiType> wikiTypes;

	private MediaDataSource mediaDataSource;

	private AttachmentDataSource attachmentDataSource;

	private WikiPageDictionary wikiPageDictionary;

	WikiPage noSuchPage;

	private Integer sessionIdExpireTime = 120;

	private int editSessionId = 0;

	private Set<Integer> editedPages;

	private Semaphore s = new Semaphore(1);

	private RenamePageWatcher renamePageWatcher;

	private Class credentialsClass;
	
	private Logger log;

	/**
	 * <p>
	 * Does ...
	 * </p>
	 * 
	 * 
	 * @param pageName
	 * @return
	 * @param wikiContext
	 */
	public String get(String pageName, WikiContext wikiContext) {
		// your code here
		return null;
	}

	/**
	 * <p>
	 * Does ...
	 * </p>
	 * 
	 * 
	 * @param pageName
	 * @param content
	 * @param wikiContext
	 * @return
	 */
	public boolean save(String pageName, String content, WikiContext wikiContext) {
		// your code here
		return false;
	}

	public boolean pageExists(String pageName) {
		return mediaDataSource.pageExists(getUid(pageName));
	}

	/**
	 * <p>
	 * Does ...
	 * </p>
	 * 
	 * 
	 * @param pageName
	 * @param type
	 * @return
	 */
	public String getContent(String pageName, String type) {
		// your code here
		return null;
	}

	/**
	 * <p>
	 * Does ...
	 * </p>
	 * 
	 * 
	 * @param pageName
	 * @return
	 * @param wikiContext
	 */
	public synchronized WikiPage getByName(String pageName,
			WikiContext wikiContext) {

		try {
			s.acquire();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		String realPageName = getUid(pageName);

		WikiPage ret = null;

		if (!mediaDataSource.preGet()) {
			log.error("Couldn't get page");
			s.release();
			return null;
		}

		if (pages.containsKey(realPageName)) {
			ret = pages.get(realPageName);
		} else {
			ret = mediaDataSource.getPage(realPageName);

			if (ret != null) {
				ret.setName(pageName);

				pages.put(realPageName, ret);
			}
		}

		if ((wikiContext != null) && (ret != null) && (!ret.isViewable())
				&& (!wikiContext.getUser().isAdmin())) {
			s.release();
			return new WikiPage(realPageName, null,
					"I am sorry, but you are not allowed to see this page", 0,
					0, new Date(), this, false, false);
		}

		if (!mediaDataSource.postGet()) {
			log.error("Problems with postGet");
		}

		if (ret != null) {
			if (wikiContext != null) {
				s.release();
				return wikiContext.process(ret);
			} else {
				s.release();
				return ret;
			}
		} else {
			try {
				ret = (WikiPage) noSuchPage.clone();
				ret.setName(pageName);
			} catch (CloneNotSupportedException cne) {
				log.error(cne);

			}
			if (wikiContext != null) {
				s.release();
				return wikiContext.process(ret);
			} else {
				s.release();
				return ret;
			}
		}
	}

	public synchronized WikiPage getByName(String pageName,
			WikiContext wikiContext, int version) {
		try {
			s.acquire();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		String realPageName = getUid(pageName);

		WikiPage ret = null;

		if (pages.containsKey(realPageName)) {
			ret = pages.get(realPageName);
		} else {
			ret = mediaDataSource.getPage(realPageName);

			ret.setName(pageName);

			pages.put(realPageName, ret);
		}

		ret = ret.getPageAtVersion(true, version);

		if (ret != null) {
			if (wikiContext != null) {
				s.release();
				return wikiContext.process(ret);
			} else {
				s.release();
				return ret;
			}
		} else {
			// return noSuchPage if there isn't any
			/*
			 * try { ret = (WikiPage) noSuchPage.clone(); } catch
			 * (CloneNotSupportedException cne) { log.error(cne); } if
			 * (wikiContext != null) { return wikiContext.process(ret); } else {
			 * return ret; }
			 */
			s.release();
			return null;
		}
	}

	/**
	 * <p>
	 * Does ...
	 * </p>
	 * 
	 * 
	 * @return
	 */
	public void init() {
		log = Logger.getLogger(WikiEngine.class);
		
		editedPages = new HashSet<Integer>();

		pages = new HashMap<String, WikiPage>();
		wikiTypes = new HashMap<String, WikiType>();

		noSuchPage = new WikiPage("", new SimpleCredentials(""),
				"There is no such page. Click on EDIT to start it.", 0, 0,
				null, this, true, true);

		loadWikiTypes();
		loadInsidePlugins();

		Properties jbosswikiprops = new Properties();

		try {
			jbosswikiprops.load(WikiEngine.class.getResourceAsStream("/"
					+ propertiesFileName));
		} catch (IOException e) {
			e.printStackTrace();
		}

		// load all properties
		if (jbosswikiprops.getProperty("defaultPage") != null) {
			defaultPage = jbosswikiprops.getProperty("defaultPage");
		}

		if (jbosswikiprops.getProperty("sessionIdExpireTime") != null) {
			sessionIdExpireTime = Integer.valueOf(jbosswikiprops
					.getProperty("sessionIdExpireTime"));
		}
		/*
		 * mediaDataSource = new FileDataSource(); attachementDataSource = new
		 * FileDataSource(); wikiPageDictionary = new FileDataSource();
		 * 
		 * 
		 */

		// LOAD data sources
		if (jbosswikiprops.getProperty("mediaDataSourceClass") != null) {
			try {
				mediaDataSource = (MediaDataSource)Class.forName(jbosswikiprops.getProperty("mediaDataSourceClass")).newInstance();
			} catch (InstantiationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			log.info("Loaded mediaDataSource: "+mediaDataSource);

		}
		else {
			//if sth went wrong
			mediaDataSource = new FileDataSource();
		}
		
		if (jbosswikiprops.getProperty("attachmentDataSourceClass") != null) {
			try {
				attachmentDataSource = (AttachmentDataSource)Class.forName(jbosswikiprops.getProperty("attachmentDataSourceClass")).newInstance();
			} catch (InstantiationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			log.info("Loaded attachmentDataSource: "+attachmentDataSource);
		}
		else {
			//if sth went wrong
			attachmentDataSource = new FileDataSource();
		}

		if (jbosswikiprops.getProperty("wikiPageDictionaryClass") != null) {
			try {
				wikiPageDictionary = (WikiPageDictionary)Class.forName(jbosswikiprops.getProperty("wikiPageDictionaryClass")).newInstance();
			} catch (InstantiationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			log.info("Loaded wikiPageDictionary: "+wikiPageDictionary);
		}
		else {
			//if sth went wrong
			wikiPageDictionary = new FileDataSource();
		}

		// set wikiEngine in classes
		mediaDataSource.setWikiEngine(this);
		attachmentDataSource.setWikiEngine(this);
		wikiPageDictionary.setWikiEngine(this);

		if (jbosswikiprops.getProperty("credentialsClass") != null) {
			try {
				credentialsClass = Class.forName(jbosswikiprops
						.getProperty("credentialsClass"));
			} catch (ClassNotFoundException e) {
				log.error("Coudln't load credentials class.",e);
			}

			if (credentialsClass != null)
				log.info("Loaded credentialsClass: "
						+ credentialsClass);
		}

		// if credentialClass didn't load or wasn't set
		if (credentialsClass == null) {
			credentialsClass = SimpleCredentials.class;
			log.info("(SIMPLE)Loaded credentialsClass: "
					+ credentialsClass);
		}

		renamePageWatcher = new RenamePageWatcher(this, wikiPageDictionary);
	}

	/**
	 * <p>
	 * Parses configuration file to get WikiTypes
	 * </p>
	 * 
	 * 
	 * @return
	 */
	private void loadWikiTypes() {
		try {
			wikiTypes.clear();

			SAXParser sp = (SAXParserFactory.newInstance()).newSAXParser();
			sp.parse(WikiTypeHandler.class
					.getResourceAsStream("/WikiTypes.xml"),
					new WikiTypeHandler(this));
		} catch (Exception se) {
			log.error("Error while parsing WikiType configuration: "
					, se);
		}
	}

	private void loadInsidePlugins() {
		insidePlugins = new HashMap<String, WikiInsidePlugin>();

		Properties props = new Properties();

		try {
			props.loadFromXML(WikiTypeHandler.class
					.getResourceAsStream("/WikiInsidePlugins.xml"));
		} catch (InvalidPropertiesFormatException e) {
			log.error("WikiInsidePlugins.xml has bad format !",e);
		} catch (IOException e) {
			log.error("There is no WikiInsidePlugins.xml file !",e);
		}

		Iterator it = props.keySet().iterator();

		while (it.hasNext()) {
			String pluginName = (String) it.next();
			log.info("Loading inside plugin: "
					+ pluginName + " class: " + props.getProperty(pluginName));

			WikiInsidePlugin wikiInsidePlugin = null;

			try {
				wikiInsidePlugin = (WikiInsidePlugin) Class.forName(
						props.getProperty(pluginName)).newInstance();

				wikiInsidePlugin.setWikiEngine(this);

				insidePlugins.put(pluginName, wikiInsidePlugin);
			} catch (InstantiationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

	}

	/**
	 * <p>
	 * Does ...
	 * </p>
	 * 
	 * 
	 * @return
	 * @param pageName
	 * @param wikiContext
	 */
	public void lock(String pageName, WikiContext wikiContext) {
		// your code here
		// return null;
	}

	/**
	 * <p>
	 * Does ...
	 * </p>
	 * 
	 * 
	 * @return
	 * @param pageName
	 * @param wikiContext
	 */
	public void unlock(String pageName, WikiContext wikiContext) {
		// your code here
		// return null;
	}

	/**
	 * <p>
	 * Does ...
	 * </p>
	 * 
	 * 
	 * @return
	 * @param path
	 * @param wikiContext
	 */
	public void getByPath(String path, WikiContext wikiContext) {
		// your code here
		// return null;
	}

	/**
	 * <p>
	 * Does ...
	 * </p>
	 * 
	 * 
	 * @return
	 * @param pageName
	 * @param wikiContext
	 */
	public void setProperty(String pageName, WikiContext wikiContext) {
		// your code here
		// return null;
	}

	/**
	 * <p>
	 * Does ...
	 * </p>
	 * 
	 * 
	 * @return
	 * @param wikiType
	 * @param userCredentials
	 */
	public void getWikiContext(String wikiType, Credentials userCredentials) {
		// your code here
		// return null;
	}

	public void addWikiType(String name, WikiType wikiType) {
		log.info("Added WikiType: " + name + " Class: "
				+ wikiType);
		wikiTypes.put(name, wikiType);
	}

	public WikiType getWikiType(String wikiTypeName) {
		// System.out
		// .println("[WIKITYPE REQUEST]: " + wikiTypes.get(wikiTypeName));
		return wikiTypes.get(wikiTypeName);
	}

	/**
	 * <p>
	 * </p>
	 * 
	 */
	public WikiEngine() {
		init();
	}

	public void refreshPage(String pageName) {
		if (pages.containsKey(pageName)) {
			pages.remove(pageName);
		}
	}

	public Map<String, Integer> findPages(String searchQuery) {
		// log.info("Starting search: " + searchQuery);
		WikiSearch wikiSearch = new WikiSearch(searchQuery);

		Map<String, Integer> resPages = new HashMap<String, Integer>();

		Set<String> pages = mediaDataSource.getAllPageNames();

		for (String name : pages) {
			WikiPage temp = getByName(name, null);

			if (temp != null) {
				int searchScore;

				// search for pages (add pagename at the beginig of each page so
				// it can be found)
				if ((searchScore = wikiSearch.matches((name + " " + temp
						.getPageContent()).toLowerCase())) > 0) {
					resPages.put(name, searchScore);
				}
			}
		}

		Object[] entryArr = resPages.entrySet().toArray();
		// Sorting the resPages by entry values (which are numbers), descending.
		Arrays.sort(entryArr, new Comparator<Object>() {
			public int compare(Object entry1, Object entry2) {
				int i1 = ((Entry<String, Integer>) entry1).getValue();
				int i2 = ((Entry<String, Integer>) entry2).getValue();

				if (i1 == i2)
					return 0;
				return (i1 < i2) ? 1 : -1;
			}
		});

		resPages = new LinkedHashMap<String, Integer>();

		for (int i = 0; i < entryArr.length; i++) {
			Entry<String, Integer> entry = (Entry<String, Integer>) entryArr[i];

			resPages.put(entry.getKey(), entry.getValue());
		}
		// ("Search completed");
		return resPages;
	}

	public Set<String> getPagesFor(String pageName) {
		Set<String> set = mediaDataSource.getPagesFor(pageName);
		Set<String> ret = new HashSet<String>();

		for (String page : set) {
			ret.add(getRealName(page));
		}

		return ret;
	}

	public String executeInsidePlugin(String pluginLink, WikiPage wikiPage,
			WikiSession wikiSession) {
		String pluginName = null;

		if (!pluginLink
				.matches("\\{[\\w]+([\\s]+[\\w]+='[\\d\\w\\s]+')*\\s*\\}")) {
			// this is not a proper inside plugin link

			return "<i>INSIDE PLUGIN ERROR: " + pluginLink
					+ " is not a proper plugin link</i>";
		}

		pluginName = pluginLink.substring(1,
				(pluginLink.indexOf(" ") != -1) ? pluginLink.indexOf(" ")
						: pluginLink.indexOf("}"));

		// get attributes
		HashMap<String, String> attributes = new HashMap<String, String>();

		Matcher matcher = Pattern.compile("[\\w]+='[\\d\\w\\s]+'").matcher(
				pluginLink);

		while (matcher.find()) {
			String attrib = pluginLink
					.substring(matcher.start(), matcher.end());

			String attribName = attrib.substring(0, attrib.indexOf("'") - 1);
			String attribValue = attrib.substring(attrib.indexOf("'") + 1,
					attrib.lastIndexOf("'"));

			attributes.put(attribName, attribValue);
		}

		if (insidePlugins.containsKey(pluginName))
			return insidePlugins.get(pluginName).process(wikiPage, wikiSession,
					attributes);
		else
			return "<i>INSIDE PLUGIN ERROR: There is no plugin: " + pluginName
					+ "</i>";
	}

	public boolean deletePage(String pageName, boolean deleteSubpages) {
		Set<String> subpages = new HashSet<String>();

		log.info("DELETE SUBPAGES: " + deleteSubpages);

		if (deleteSubpages) {
			subpages = mediaDataSource.getPagesFor(pageName);
		}

		if (attachmentDataSource.getAttachmentsSet(getByName(pageName, null)) != null) {
			if (!attachmentDataSource.deleteAttachments(pageName)) {
				log.error("Couldn't delete attachments of page "
						+ pageName);
				return false;
			}
		}

		boolean ret = true;

		if ((deleteSubpages) && (subpages.size() > 0)) {

			for (String subPage : subpages) {
				if (!deletePage(subPage, deleteSubpages)) {
					System.err
							.println("[WikiEngine]: Problems with deleteing subpage: "
									+ subPage);
					ret = false;
				}
			}

		}

		if (mediaDataSource.deletePage(pageName)) {

			if (pages.containsKey(pageName)) {
				pages.remove(pageName);
			}

			log.info("Finishing deleting of page " + pageName
					+ " with ret = " + ret);

			return ret;
		} else {
			log.error("Problems with deleting page " + pageName);
			return false;
		}
	}

	public String getRealName(String uid) {
		String[] pageTokens = uid.split("/");

		// log.info("Looking at page: (realName) "+uid);

		if (pageTokens.length > 1) {
			if (!wikiPageDictionary.getRealName(uid).equals(uid)) {

				// this page is in dictionary
				return wikiPageDictionary.getRealName(uid);
			} else {
				// log.info("checking parts");
				// look if parts of this page are in dictionary

				for (int i = pageTokens.length - 1; i > 0; i--) {
					String page = "";
					String suffix = "";
					for (int j = 0; j < i; j++) {
						page += "/" + pageTokens[j];
					}

					// build suffix

					for (int j = i; j < pageTokens.length; j++) {
						suffix += "/" + pageTokens[j];
					}

					page = page.substring(1);

					// log.info("Looking at page: (realName) "+page);

					if (!wikiPageDictionary.getRealName(page).equals(page)) // this
						// page
						// is
						// in
						// dictionary
						return wikiPageDictionary.getRealName(page) + suffix;

				}

				return uid;
			}
		} else
			return wikiPageDictionary.getRealName(uid);
	}

	public String getUid(String realName) {
		String[] pageTokens = realName.split("/");

		// log.info("Looking at page: (uid) "+realName);

		if (pageTokens.length > 1) {
			if (!wikiPageDictionary.getUid(realName).equals(realName)) {
				// this page is in dictionary
				return wikiPageDictionary.getUid(realName);
			} else {
				// look if parts of this page are in dictionary

				for (int i = pageTokens.length - 1; i > 0; i--) {
					String page = "";
					String suffix = "";
					for (int j = 0; j < i; j++) {
						page += "/" + pageTokens[j];
					}

					// build suffix

					for (int j = i; j < pageTokens.length; j++) {
						suffix += "/" + pageTokens[j];
					}

					page = page.substring(1);

					// log.info("Looking at page: (uid) "+page);

					if (!wikiPageDictionary.getUid(page).equals(page)) // this
						// page
						// is
						// in
						// dictionary
						return wikiPageDictionary.getUid(page) + suffix;
				}

				return realName;
			}
		} else
			return wikiPageDictionary.getUid(realName);
	}

	public synchronized void rename(String uid, String newName)
			throws WikiException {
		if (newName.indexOf("/") != -1) {
			throw new PageRenamingException(
					"You can rename only page/subpage name. / is not allowed.");
		}

		if (uid.indexOf("/") != -1) {
			// renaming subpage - create aprop. newName
			newName = uid.substring(0, uid.lastIndexOf("/") + 1) + newName;
		}

		if (!arePagesEdited()) {
			wikiPageDictionary.rename(uid, newName);
		} else {
			// some pages are edited now - execute watcher
			renamePageWatcher.rename(uid, newName);

			if (!renamePageWatcher.isRuning()) {
				renamePageWatcher.start();
			}
		}
	}

	public boolean uidInDictionary(String uid) {
		return wikiPageDictionary.uidInDictionary(uid);
	}

	public boolean realNameInDictionary(String realName) {
		return wikiPageDictionary.realNameInDictionary(realName);
	}

	public boolean deleteAttachment(String pageName, String attName) {
		return attachmentDataSource.deleteAttachment(pageName, attName);
	}

	public boolean deleteAttachments(String pageName) {
		return attachmentDataSource.deleteAttachments(pageName);
	}

	public Set<String> getAttachmentsSet(WikiPage page) {
		return attachmentDataSource.getAttachmentsSet(page);
	}

	public int getLastAttachmentVersion(String pageName, String attachmentName) {
		return attachmentDataSource.getLastAttachmentVersion(pageName,
				attachmentName);
	}

	public WikiAttachment getAttachment(String pageName,
			String attachementName, int version) {
		return attachmentDataSource.getAttachment(pageName, attachementName,
				version);
	}

	public WikiAttachment getAttachment(String pageName, String attachementName) {
		return attachmentDataSource.getAttachment(pageName, attachementName);
	}

	public void addAttachment(File attFile, String attName, WikiPage page,
			String user) {
		attachmentDataSource.addAttachment(attFile, attName, page, user);
	}

	public Set<String> getAllPageNames() {
		Set<String> set = mediaDataSource.getAllPageNames();
		Set<String> translatedSet = new TreeSet<String>();

		for (String page : set) {
			translatedSet.add(getRealName(page));
		}

		return translatedSet;
	}

	public boolean preSave() {
		return mediaDataSource.preSave();
	}

	public boolean savePage(WikiPage page, String languageCode) {
		WikiType postEditProcess = getWikiType("postedit");

		if (postEditProcess != null) {
			// there are some plugins for postediting a page
			page = postEditProcess.process(page, null);
		}

		// put processed page to pages map
		pages.put(page.getName(), page);

		return mediaDataSource.savePage(getUid(page.getName()), page,
				languageCode);
	}

	public boolean postSave() {
		return mediaDataSource.postSave();
	}

	public boolean preGet() {
		return mediaDataSource.preGet();
	}

	public boolean postGet() {
		return mediaDataSource.postGet();
	}

	public WikiPage getPageAtVersion(WikiPage originPage, boolean loadContent,
			String languageCode, int version) {
		return mediaDataSource.getPageAtVersion(originPage, loadContent,
				languageCode, version);
	}

	public WikiPage getPageAtVersion(WikiPage originPage, boolean loadContent,
			int version) {
		return mediaDataSource.getPageAtVersion(originPage, loadContent,
				version);
	}

	public void getContentAtVersion(WikiPage page, boolean loadContent,
			int version) {
		mediaDataSource.getContentAtVersion(page, loadContent, version);
	}

	public int getPageMod(String pageName) {
		return mediaDataSource.getPageMod(pageName);
	}

	public void setPageMod(String pageName, int mods) {
		mediaDataSource.setPageMod(pageName, mods);
	}

	public boolean deletePage(String pageName) {
		return mediaDataSource.deletePage(pageName);
	}

	public long getAttachmentSize(String pageName, String attachmentName,
			int version) {
		return attachmentDataSource.getAttachmentSize(pageName,
				attachmentName, version);
	}

	public synchronized int getEditSessionId() {
		return editSessionId++;
	}

	public Integer getSessionIdExpireTime() {
		return sessionIdExpireTime;
	}

	public boolean arePagesEdited() {
		return (editedPages.size() > 0);
	}

	public void endEditing(Integer editSessionId) throws PageNotEditedException {
		if (!editedPages.contains(editSessionId)) {
			throw new PageNotEditedException("Page with sessionid "
					+ editSessionId + " is not beeing edited.");
		}

		editedPages.remove(editSessionId);
	}

	public void addEditing(Integer editSessionId) {
		editedPages.add(editSessionId);
	}

	public Semaphore getS() {
		return s;
	}

	public HashMap<String, String> getPagesWaiting() {
		return renamePageWatcher.getPagesWaiting();
	}

	public Credentials getCredentialsInstance(RenderRequest rReq) {
		Credentials cred = null;

		try {
			cred = (Credentials) credentialsClass.newInstance();
			cred.setRenderRequest(rReq);
		} catch (InstantiationException e) {
			log.error("Couldn't get instance of class "
					+ credentialsClass.getName(),e);
		} catch (IllegalAccessException e) {
			log.error(e);
		}

		return cred;
	}

	public Credentials getCredentialsInstance(ActionRequest aReq) {
		Credentials cred = null;

		log.debug("Getting instance of " + credentialsClass.getName());

		try {
			cred = (Credentials) credentialsClass.newInstance();
			cred.setActionRequest(aReq);
		} catch (InstantiationException e) {
			log.error("Couldn't get instance of class "
					+ credentialsClass.getName(),e);
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		}

		return cred;
	}
}
