/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki;

import java.io.File;
import java.io.Serializable;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import org.jboss.logging.Logger;

import org.jboss.wiki.exceptions.EditSessionExpired;
import org.jboss.wiki.exceptions.PageNotEditedException;

/**
 * <p>
 * </p>
 * 
 */
/**
 * 
 * @author <a href="mailto:dsicore@jboss.org">Damon Sicore</a>
 * @author <a href="mailto:tomasz.szymanski@jboss.com">Tomasz Szymanski</a>
 * @version $Id$
 */
public class WikiPage implements Serializable, WikiProperties, Cloneable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/* number of minutes after which page lock will expire */
	private static final int LOCK_TIME = 30;

	private Date lockTime = null;

	private String lockUser;

	private long length = 0;
	
	private Set<Integer> expiredSessionIds;
	
	private HashMap<Integer, EditSessionWatcher> editSessionWatchers;

	/**
	 * <p>
	 * Represents ...
	 * </p>
	 * 
	 */
	private String name;
	
	private Logger log;

	/**
	 * <p>
	 * Represents ...
	 * </p>
	 * 
	 * WikiPage This class is _abstract_ and has to be extended by the class
	 * that we're willing to show our content
	 * 
	 * functions to be extended: WikiPage(...) - constructor getAttributes() -
	 * gets set of atributes to be send to the appropriate JSP page
	 * saveThisPage(data:Object) - saves next version of this page with new data
	 * getJSPPath() - gets apropriate JSP page to show plugin
	 * getDataProviderName - gets name of the plugin _must_ be the same as in
	 * the plugin name in plugin.xml
	 */
	private Credentials lastAuthor;

	/**
	 * <p>
	 * Represents ...
	 * </p>
	 * 
	 */
	private int lastVersion;

	private int version;

	/**
	 * <p>
	 * Represents ...
	 * </p>
	 * 
	 */
	private Date editDate;

	/**
	 * <p>
	 * Represents ...
	 * </p>
	 * 
	 */

	/**
	 * <p>
	 * </p>
	 * 
	 */
	private WikiEngine wikiEngine;

	/**
	 * <p>
	 * </p>
	 * 
	 */
	public WikiEngine wikiEngine_1;

	private String pageContent;

	private boolean editable;

	private boolean viewable;

	public WikiPage(String pageName, Credentials pageAuthor,
			String pageContent, int pageLastVersion, int thisVersion,
			Date editDate, WikiEngine wikiEngine, boolean viewable,
			boolean editable) {
		name = pageName;
		lastAuthor = pageAuthor;
		this.pageContent = pageContent;
		lastVersion = pageLastVersion;
		version = thisVersion;
		this.editDate = editDate;
		this.wikiEngine = wikiEngine;
		this.viewable = viewable;
		this.editable = editable;
		expiredSessionIds = new HashSet<Integer>();
		editSessionWatchers = new HashMap<Integer, EditSessionWatcher>();
		log = Logger.getLogger(WikiPage.class);
	}

	/**
	 * <p>
	 * Does ...
	 * </p>
	 * 
	 * 
	 * @return
	 */
	public HashMap getAttributes() {
		// your code here
		return null;
	}

	/**
	 * <p>
	 * Does ...
	 * </p>
	 * 
	 */
	public synchronized boolean save(Integer editActionId) throws EditSessionExpired{

		if (expiredSessionIds.contains(editActionId)) {
			throw new EditSessionExpired("Your edit session have expired. Please edit page once again !");
		}
		
		if (editSessionWatchers.get(editActionId) != null) {
			editSessionWatchers.get(editActionId).stopWatcher();
			editSessionWatchers.remove(editActionId);
		}
		
		if (!wikiEngine.preSave()) {
			log.error("Couldn't save page");
			return false;
		}
		boolean ret = wikiEngine.savePage(this, null);
		boolean postSave;
		
		if (postSave = !wikiEngine.postSave()) {
			log.error("Problems with postSave");
		}
		
		return ret || postSave;
	}

	/**
	 * <p>
	 * Does ...
	 * </p>
	 * 
	 * 
	 * @return
	 */
	public String getPath() {
		// your code here
		return null;
	}


	/**
	 * <p>
	 * Does ...
	 * </p>
	 * 
	 * 
	 * @param version
	 * @return
	 */
	public boolean updateToVersion(int version) {
		// your code here
		return false;
	}

	/**
	 * <p>
	 * Does ...
	 * </p>
	 * 
	 * 
	 * @return
	 * @param version
	 */
	public WikiPage getPageAtVersion(boolean loadContent, int version) {
		if (version == lastVersion) {
			return this;
		} else {
			return wikiEngine.getPageAtVersion(this, loadContent, version);
		}
	}

	/**
	 * <p>
	 * Does ...
	 * </p>
	 * 
	 * 
	 * @return
	 */
	public String getContent() {
		return getPageContent();
	}

	/**
	 * <p>
	 * Does ...
	 * </p>
	 * 
	 * 
	 * @return
	 */
	public MediaDataSource getMediaDataSource() {
		// your code here
		return null;
	}

	/**
	 * <p>
	 * Does ...
	 * </p>
	 * 
	 * 
	 * @return
	 * @param text
	 */
	public void setWikiText(int text) {
		// your code here
		// return null;
	}

	public void showPage() {
		log.info("Page name: " + getName());
		log.info("Page author: " + getLastAuthor());
		log.info("Page version: " + getLastVersion());
		log.info("Page editdate: " + getEditDate());
		log.info("Page Content: \n" + getContent());
	}

	/**
	 * <p>
	 * </p>
	 * 
	 */
	public MediaInternalization mediaInternalization;

	public java.util.Date getEditDate() {
		return editDate;
	}

	public void setEditDate(java.util.Date editDate) {
		this.editDate = editDate;
	}

	public Credentials getLastAuthor() {
		return lastAuthor;
	}

	public void setLastAuthor(Credentials lastAuthor) {
		this.lastAuthor = lastAuthor;
	}

	public int getLastVersion() {
		return lastVersion;
	}

	public void setLastVersion(int lastVersion) {
		this.lastVersion = lastVersion;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getPageContent() {
		if (pageContent == null) {
			// page wasn't loaded with conent - lets do it
			wikiEngine.getContentAtVersion(this, true, version);
		}
		return pageContent;
	}

	public void setPageContent(String pageContent) {
		this.pageContent = pageContent;
	}

	@Override
	public Object clone() throws CloneNotSupportedException {
		WikiPage clonedPage = new WikiPage(getName(), getLastAuthor(),
				getPageContent(), getLastVersion(), getVersion(),
				getEditDate(), wikiEngine,isViewable(),isEditable());

		clonedPage.setLength(getLength());

		return clonedPage;
	}

	public void unlock(Credentials user, Integer editSessionId) {
		log.info("UNLOCK: " + user.getName() + " save user: "
				+ lockUser);
		
		if (editSessionWatchers.get(editSessionId) != null) {
			editSessionWatchers.get(editSessionId).stopWatcher();
			editSessionWatchers.remove(editSessionId);
		}
		
		try {
			wikiEngine.endEditing(editSessionId);
		} catch (PageNotEditedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		if ((user.getName()).equals(lockUser)) {
			log.info("Unlocking page");
			lockTime = null;
			lockUser = null;
		}
	}

	public Date lock(Credentials user, Integer editSessionId) {
		Date now = new Date();
		Date expDate = null;
		Date retDate = lockTime;

		if (this.lockTime != null) {
			// lets calculete expire date of current lock

			Calendar calExpireDate = Calendar.getInstance();
			calExpireDate.setTime(this.lockTime);
			calExpireDate.add(Calendar.MINUTE, LOCK_TIME);
			expDate = calExpireDate.getTime();

			if (expDate.before(now)) {
				// lock expired. take it out
				lockTime = null;
				lockUser = null;
			}
		}

		if (this.lockTime == null) {
			log.info("LOCK: " + user.getName());

			// page isn't locked or lock expired. Lets lock it.
			this.lockTime = now;
			this.lockUser = user.getName();
			log.info("LOCK: " + lockUser);
			retDate = null;
		}
		
		// Add EditSessionWatcher
		wikiEngine.addEditing(editSessionId);
		EditSessionWatcher eSW = new EditSessionWatcher(editSessionId, wikiEngine.getSessionIdExpireTime(), this);
		eSW.start();
		
		editSessionWatchers.put(editSessionId, eSW);

		return retDate;
	}

	public String getLockUser() {
		return lockUser;
	}

	public int getVersion() {
		return version;
	}

	public void setVersion(int version) {
		this.version = version;
	}

	public long getLength() {
		if ((length == 0) && (getPageContent().length() != 0)) {
			length = getPageContent().length();
		}

		return length;
	}

	public void setLength(long length) {
		this.length = length;
	}

	public void addAttachement(File attFile, String attName, String user) {
		wikiEngine.addAttachment(attFile, attName, this, user);
	}

	public Set<String> getAttachementsSet() {
		return wikiEngine.getAttachmentsSet(this);
	}

	public boolean isEditable() {
		return editable;
	}

	public void setEditable(boolean editable) {
		this.editable = editable;
		
		wikiEngine.setPageMod(getName(), ((isViewable()) ? MediaDataSource.VIEWABLE : 0) + ((isEditable()) ? MediaDataSource.EDITABLE : 0));
	}

	public boolean isViewable() {
		return viewable;
	}

	public void setViewable(boolean viewable) {
		this.viewable = viewable;
		
		wikiEngine.setPageMod(getName(), ((isViewable()) ? MediaDataSource.VIEWABLE : 0) + ((isEditable()) ? MediaDataSource.EDITABLE : 0));
	}
	
	public void addExpiredSessionId(Integer id) throws PageNotEditedException {
		expiredSessionIds.add(id);
		wikiEngine.endEditing(id);
	}
}
