/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki.plugins;

import java.util.Set;

import org.jboss.wiki.Credentials;
import org.jboss.wiki.WikiEngine;
import org.jboss.wiki.WikiPage;
import org.jboss.wiki.WikiPlugin;
import org.jboss.wiki.WikiSession;

public class AdminConsolePlugin extends WikiPlugin {

	private boolean SHOWDELETE = true;

	@Override
	public WikiPage process(WikiPage wikiPage, WikiSession wikiSession) {
		Credentials credentials = (Credentials) wikiSession
				.getAttribute("credentials");

		if (credentials.isAdmin()) {
			String actionURL = (String) wikiSession.getAttribute("actionURL");
			String errorMsg = "";

			boolean editable = wikiPage.isEditable();
			boolean viewable = wikiPage.isViewable();

			if (wikiSession.getAttribute("var1") != null) {

				if (wikiSession.getAttribute("var1").equals("chmod")) {
					WikiPage editingPage = wikiEngine.getByName(wikiPage
							.getName(), null);

					if (wikiSession.getAttribute("var2") != null) {
						editingPage.setEditable(Boolean
								.valueOf((String) wikiSession
										.getAttribute("var2")));
						editable = editingPage.isEditable();
					}
					if (wikiSession.getAttribute("var3") != null) {
						editingPage.setViewable(Boolean
								.valueOf((String) wikiSession
										.getAttribute("var3")));
						viewable = editingPage.isViewable();
					}
				} else if ((SHOWDELETE)
						&& (wikiSession.getAttribute("var1")
								.equals("deletePage"))) {
					boolean deleteSubpages = (wikiSession.getAttribute("var3") != null) ? Boolean
							.parseBoolean((String) wikiSession
									.getAttribute("var3"))
							: false;

					if (wikiSession.getAttribute("var2") != null) {
						String nameToDelete = (String) wikiSession
								.getAttribute("var2");

						if (!nameToDelete.equals(WikiEngine.defaultPage)) {
							if (!wikiEngine.deletePage(nameToDelete,
									deleteSubpages)) {
								errorMsg += "Couldn't delete whole or parts of page: "
										+ nameToDelete
										+ " or it's subpages (see log for moreinfo)";
							}
						} else {
							errorMsg += "You can't delete "
									+ WikiEngine.defaultPage + " page !";
						}
					}
				} else if ((SHOWDELETE)
						&& (wikiSession.getAttribute("var1")
								.equals("deleteAtt"))) {
					String attName = (wikiSession.getAttribute("var2") != null) ? (String) wikiSession
							.getAttribute("var2")
							: null;

					if (attName != null) {
						if (!wikiEngine.deleteAttachment(wikiPage.getName(),
								attName)) {
							errorMsg = "There was problem with deleting "
									+ attName + " from page "
									+ wikiPage.getName()
									+ " (see log for more info)";
						}
					} else {
						errorMsg = "Attachment to delete isn't specified";
					}
				}

			}

			StringBuilder adminConsole = new StringBuilder(wikiPage
					.getPageContent());

			adminConsole.append("\n<hr><h3>Admin console</h3>\n");
			adminConsole.append("Go to <a href=\"").append(actionURL).append(
					"&type=admin").append(
					"\">ADMIN GENERAL CONSOLE</a><br />\n");
			if (errorMsg.length() > 0) {
				adminConsole.append("<i>" + errorMsg + "</i><br />\n");
			}

			if (SHOWDELETE) {
				Set<String> attSet = wikiEngine.getAttachmentsSet(wikiPage);

				if (attSet != null) {
					adminConsole
							.append("<h4>Attachments:</h4>\n<table border=\"0\">");

					for (String att : attSet) {
						adminConsole.append("<tr><td>").append(att).append(
								"</td><td><a href=\"").append(actionURL)
								.append("&page=").append(wikiPage.getName())
								.append("&var1=deleteAtt").append("&var2=")
								.append(att).append("\" >Delete</a></td></tr>");
					}

					adminConsole.append("</table><br />\n");
				}

				adminConsole.append("<h4>Page managment:</h4>");

				if (!wikiPage.getName().equals(WikiEngine.defaultPage)) {

					// delete page button
					adminConsole.append("<a href=\"").append(actionURL).append(
							"&page=Main").append("&var1=deletePage").append(
							"&var2=").append(wikiPage.getName()).append(
							"&var3=false").append(
							"\" >DELETE THIS PAGE</a><br />\n");

					// delete page with subpages button
					adminConsole
							.append("<a href=\"")
							.append(actionURL)
							.append("&page=Main")
							.append("&var1=deletePage")
							.append("&var2=")
							.append(wikiPage.getName())
							.append("&var3=true")
							.append(
									"\" >DELETE THIS PAGE WITH SUBPAGES</a><br />\n<br />\n");
				}
			}
			// switch editable button
			adminConsole.append("This page ").append(
					(editable) ? "is" : "isn't").append(" editable ");
			adminConsole.append("<a href=\"").append(actionURL)
					.append("&page=").append(wikiPage.getName()).append("&var1=chmod").append("&var2=")
					.append(!editable).append("&var3=").append(viewable)
					.append("\" >SWITCH IT ").append((editable) ? "NOT" : "")
					.append(" EDITABLE</a><br />\n");

			// switch viewable button
			adminConsole.append("This page ").append(
					(viewable) ? "is" : "isn't").append(" viewable ");

			adminConsole.append("<a href=\"").append(actionURL)
					.append("&page=").append(wikiPage.getName()).append(
							"&var1=chmod").append("&var2=").append(editable)
					.append("&var3=").append(!viewable)
					.append("\" >SWITCH IT ").append((viewable) ? "NOT" : "")
					.append(" VIEWABLE</a><br />\n");

			wikiPage.setPageContent(adminConsole.toString());
		}
		return wikiPage;
	}

	@Override
	public void init() {

	}

}
