/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki.plugins;

import java.util.Date;
import java.util.HashMap;
import java.util.Set;

import org.jboss.wiki.Credentials;
import org.jboss.wiki.WikiPage;
import org.jboss.wiki.WikiPlugin;
import org.jboss.wiki.WikiSession;
import org.jboss.wiki.exceptions.WikiException;

public class AdminRenamingPlugin extends WikiPlugin {

	public final String RENAME_CAPTION = "Rename"; 
	@Override
	public WikiPage process(WikiPage wikiPage, WikiSession wikiSession) {
		Credentials credentials = (Credentials)wikiSession.getAttribute("credentials");
		String actionURL = (String)wikiSession.getAttribute("actionURL");
		StringBuilder adminContent = new StringBuilder("<h2><a href=\"").append(actionURL).append("&type=admin \">Admin</a></h2><br />\n");
		
		if (credentials.isAdmin()) {
			
			adminContent.append("<h3>Renaming:</h3>\n<br />\n");
			
			if((wikiSession.getAttribute("var1") != null)&&(wikiSession.getAttribute("var1").equals(RENAME_CAPTION))) {
				if ((wikiSession.getAttribute("var2") != null)&&(wikiSession.getAttribute("var3") != null)) {
					String oldName = (String)wikiSession.getAttribute("var2");
					String uid = wikiEngine.getUid(oldName);
					String newName = HTMLTranslatorParts.cleanLink((String)wikiSession.getAttribute("var3"));
					
					try {
						wikiEngine.rename(uid, newName);
						adminContent.append("Added renaming from page: ").append(oldName).append(" to page: ").append(newName).append("<br /><br />\n");
					} catch (WikiException e) {
						adminContent.append("Couldn't rename page because: ").append(e.getMessage()).append("<br /><br />\n");
						e.printStackTrace();
					}
				}
			}
			
			HashMap<String,String> pagesWaiting = wikiEngine.getPagesWaiting();
			
			if (pagesWaiting.size() > 0) {
				adminContent.append("Pages waiting for rename:<br />\n");
				adminContent.append("<table border=\"0\">\n");
				adminContent.append("<tr><td><b>Uid</b></td><td><b>Real name</b></td></tr>");
				for (String uid : pagesWaiting.keySet()) {
					adminContent.append("<tr><td>").append(uid).append("</td><td>").append(pagesWaiting.get(uid)).append("</td></tr>");
				}
				adminContent.append("</table>\n");
			}
			
			Set<String> allPages = wikiEngine.getAllPageNames();
			
			adminContent.append("<form method=\"POST\" action=\"").append(actionURL).append("&type=admin \" >\n");
			adminContent.append("<select name=\"var2\" >\n");
			
			for (String pageName : allPages) {
				adminContent.append("<option>").append(pageName).append("</option>\n");
			}
			adminContent.append("</select>\n");
			adminContent.append(" rename to ").append("<input type=\"text\" name=\"var3\" />").append("<input type=\"submit\" name=\"var1\" value=\"").append(RENAME_CAPTION).append("\" />\n");
			adminContent.append("</form>\n");
		}
		else {
			adminContent.append("<h3>").append("Sorry you don't have permission to see this page").append("</h3>");
		}
		
		return new WikiPage("AdminPage", credentials, adminContent.toString(), 0, 0,new Date(), wikiEngine, false, false);
	}

	@Override
	public void init() {
		// TODO Auto-generated method stub

	}

}
