/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki.plugins;

import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.jboss.wiki.WikiPage;
import org.jboss.wiki.WikiPlugin;
import org.jboss.wiki.WikiSession;

public class FriendlyLinkPlugin extends WikiPlugin {

	private String url;
	private String pattern;
	
	private String translateLinks(String content) {
		//<a href="http://localhost:8080/portal/index.html?ctrl:id=window.default.WikiPortletWindow&ctrl:type=action&page=JBossWiki">
		String newContent = content;
		Vector<String> replaces = new Vector<String>();
		
		Matcher matcher = Pattern.compile(pattern).matcher(newContent);
		
		while (matcher.find()) {
			String link = content.substring(matcher.start(), matcher.end());
			replaces.add("<a href=\""+url+link.substring(link.lastIndexOf("&page=")+("&page=".length()), link.length()-2)+"\" >");
		}
		
		for (int i = 0; i < replaces.size(); i++) {
			matcher = Pattern.compile(pattern).matcher(newContent);
			newContent = matcher.replaceFirst(replaces.get(i));
		}
		
		//System.out.println(content);
		
		return newContent;
	}
	
	@Override
	public WikiPage process(WikiPage wikiPage, WikiSession wikiSession) {
		url = (String)wikiSession.getAttribute("hostURL")+"/wiki/";
		
		pattern = "<a href=\""+(((String)wikiSession.getAttribute("hostURL")+(String)wikiSession.getAttribute("actionURL")).replaceAll("\\.","\\\\.").replaceAll("\\?", "\\\\?"))+"&page=\\w+(/\\w+)*\">";
		
		//System.out.println(pattern);
		
		WikiPage newPage = null;

		try {
			newPage = (WikiPage) wikiPage.clone();
		} catch (CloneNotSupportedException cl) {
			System.err.println(cl);
		}

		newPage.setPageContent(translateLinks(newPage.getPageContent()));

		return newPage;
	}

	@Override
	public void init() {
		
	}

}
