/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki.plugins;

import java.util.Date;

import org.jboss.wiki.WikiPage;

import org.jboss.wiki.WikiPlugin;
import org.jboss.wiki.WikiSession;

public class RDFPlugin extends WikiPlugin {

	private DifferenceEngine differenceEngine;

	private void addItem(StringBuilder rdfFeed, String title, String link,
			String description, Date date) {
		rdfFeed.append("<item rdf:about=\"").append(link).append("\" >\n");
		rdfFeed.append("<title>").append(title).append("</title>\n");
		rdfFeed.append("<link>").append(link).append("</link>\n");
		rdfFeed.append("<description>").append(description).append(
				"</description>\n");
		rdfFeed.append("<dc:date>").append(date.toString()).append("</dc:date>\n");
		rdfFeed.append("</item>\n");
	}

	@Override
	public WikiPage process(WikiPage wikiPage, WikiSession wikiSession) {
		StringBuilder rdfFeed = new StringBuilder(
				"<!-- name=\"generator\" content=\"JBoss Wiki RDFPlugin\" -->\n");
		rdfFeed.append("<rdf:RDF>\n");
		rdfFeed.append("<channel rdf:about=\"").append((String) wikiSession.getAttribute("hostURL")
				+ "/wiki/" + wikiPage.getName()).append("\">\n");
		rdfFeed.append("<title>JBoss Forge Wiki: "+wikiPage.getName()+"</title>\n");
		rdfFeed.append("<link>" + (String) wikiSession.getAttribute("hostURL")
				+ "/wiki/" + wikiPage.getName() + "</link>\n");
		rdfFeed.append("<description>JBoss Wiki: " + wikiPage.getName()
				+ "</description>\n");
		rdfFeed.append("<dc:language>en</dc:language>\n");
		rdfFeed.append("<items>\n<rdf:Seq>\n");
		
		StringBuilder items = new StringBuilder();
		
		for (int i = wikiPage.getLastVersion(); i > 0; i--) {
			String content;
			
			if (i > 1) {
					content = differenceEngine.makeDiff(wikiPage.getPageAtVersion(true, i).getPageContent(), wikiPage.getPageAtVersion(true, i-1).getPageContent()).replaceAll("&", "&amp;").replaceAll("<","&lt;").replaceAll(">","&gt;").replaceAll("\n","<br />\n");
			}
			else {
				content = "This is first version of the page so it doesn't have diff yet";
			}
			
			String url = (String) wikiSession.getAttribute("hostURL")
			+ getProperty("wikiHome").replaceAll("&", "&amp;")
			+ "&amp;page=" + wikiPage.getName()
			+ "&amp;version=" + i;
			
			addItem(items, wikiPage.getName() + " rev: " + i,
					url, content, wikiPage.getEditDate());
			
			rdfFeed.append("<rdf:li rdf:resource=\"").append(url).append("\" />\n");
		}

		rdfFeed.append("</rdf:Seq>\n</items>\n</channel>\n").append(items).append("</rdf:RDF>\n");

		wikiPage.setPageContent(rdfFeed.toString());

		return wikiPage;
	}

	@Override
	public void init() {
		differenceEngine = new DifferenceEngine("ISO-8859-1");
	}

}
