/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki.plugins;

import java.util.LinkedList;
import java.util.Queue;
import java.util.Stack;

import org.jboss.wiki.WikiPage;
import org.jboss.wiki.WikiPlugin;
import org.jboss.wiki.WikiSession;

public class TranslateUids extends WikiPlugin {

	private Stack<Integer> begins = new Stack<Integer>();

	private Stack<Integer> ends = new Stack<Integer>();

	private Stack<String> replaces = new Stack<String>();

	private String PRE_START = "{{{";

	private String PRE_END = "}}}";

	@Override
	public WikiPage process(WikiPage wikiPage, WikiSession wikiSession) {

		wikiPage.setPageContent(getNewContent(wikiPage.getContent()));

		return wikiPage;
	}

	public String getNewContent(String content) {
		StringBuffer sBuf = new StringBuffer(content);
		Queue<Integer> queue = new LinkedList<Integer>();

		Integer lastCommStart = -3;
		Integer partBegin = 0;

		// get all PRE starting {{{
		while ((lastCommStart = sBuf.indexOf(PRE_START, lastCommStart + 3)) != -1) {
			queue.offer(lastCommStart);
		}

		lastCommStart = queue.poll();

		if (lastCommStart == null) {
			// there are no comments
			parsePart(sBuf, 0, sBuf.length());
		} else {
			while (lastCommStart != null) {

				parsePart(sBuf, partBegin, lastCommStart);

				partBegin = sBuf.indexOf(PRE_END, lastCommStart);

				if (partBegin != -1) {
					partBegin += PRE_END.length();

					while ((lastCommStart != null)
							&& (lastCommStart < partBegin)) {
						lastCommStart = queue.poll();
					}

					if (lastCommStart == null) {
						lastCommStart = sBuf.length();
					}
				} else {
					lastCommStart = null;
				}
			}
		}

		while (begins.size() > 0) {

			int begin;

			sBuf.delete(begin = begins.pop(), ends.pop());

			sBuf.insert(begin, replaces.pop());
		}

		return sBuf.toString();
	}

	protected void parsePart(StringBuffer sBuf, int start, int end) {

		boolean insideLink = false;
		Integer wordBegin = null;
		Integer wordEnd = null;

		for (int i = start; i < end; i++) {
			// System.out.print(i+":'"+sBuf.charAt(i)+"'");

			if (sBuf.charAt(i) == '[') {
				// if it is false - make it true
				insideLink = (insideLink || true);
				wordBegin = i + 1;
				wordEnd = i + 1;
			} else if (sBuf.charAt(i) == '|') {
				if (!insideLink) {
					wordBegin = null;
					wordEnd = null;
				} else {
					wordBegin = i + 1;
				}

			} else if (sBuf.charAt(i) == ']') {

				if ((insideLink) && (wordBegin != null) && (wordBegin != i)) {
					wordEnd++;

					// System.out.println("End of link:
					// "+sBuf.substring(wordBegin, wordEnd));

					// if word is GT 0

					if (!HTMLTranslatorParts.isExternalLink(sBuf.substring(
							wordBegin, wordEnd))) {
						String link = HTMLTranslatorParts.cleanLink(sBuf
								.substring(wordBegin, wordEnd));

						if (link != changeName(link)) {
							begins.add(wordBegin);
							ends.add(wordEnd);
							replaces.add(changeName(link));
						}
					}
					wordBegin = null;
					wordEnd = null;
					insideLink = false;
				}
			} else if (isPartOfWord(sBuf.charAt(i))) {
				if (wordBegin == null) {
					wordBegin = i;
				}

				wordEnd = i;
			} else if (Character.isWhitespace(sBuf.charAt(i))) {
				if ((!insideLink) && (wordBegin != null)) {
					String link = null;

					if ((link = HTMLTranslatorParts.parseCamelCase(sBuf
							.substring(wordBegin, wordEnd))) != null) {
						if (link != changeName(link)) {
							begins.add(wordBegin);
							ends.add(wordEnd);
							replaces.add(changeName(link));
						}

						wordBegin = null;
						wordEnd = null;
					}
				} else {
					wordEnd = i;
				}
			} else {
				if (!insideLink) {
					wordBegin = null;
					wordEnd = null;
				} else {
					wordEnd = i;
				}
			}
		}

		if (wordBegin != null) {
			// there was not closed link ([...) parse what stayed unparsed
			// (maybe some CamelCase links)
			if (wordBegin != start)
				parsePart(sBuf, wordBegin, end);
		}
	}

	protected boolean isPartOfWord(char ch) {
		return ((Character.isLetter(ch)) || (Character.isDigit(ch)));
	}

	@Override
	public void init() {

	}

	protected String changeName(String link) {
		return wikiEngine.getRealName(link);
	}
}
