/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki.test;

import org.jboss.wiki.WikiContext;
import org.jboss.wiki.WikiPage;
import org.jboss.wiki.exceptions.EditSessionExpired;
import org.jboss.wiki.exceptions.WikiException;
import org.jboss.wiki.plugins.ReTranslateUids;

public class DictionaryTest extends WikiTest {
	
	public void testDictionary() throws WikiException{
		
		WikiContext wikiContext = new WikiContext(plainUser, wikiEngine.getWikiType("html"), testWikiSession);
		
		String page1 = createTestPage(plainUser, "1 page number 1");
		String page2 = createTestPage(plainUser, "2 page number 2");
		
		WikiPage wikiPage1 = wikiEngine.getByName(page1, null);
		WikiPage wikiPage2 = wikiEngine.getByName(page2, null);
		
		
		// set apropriate content of pages
		wikiPage1.setPageContent("1 ["+page2+"]");
		wikiPage1.setLastVersion(wikiPage1.getLastVersion()+1);
		wikiPage1.save(wikiEngine.getEditSessionId());
		
		wikiPage2.setPageContent("1 ["+page2+"]");
		wikiPage2.setLastVersion(wikiPage2.getLastVersion()+1);
		wikiPage2.save(wikiEngine.getEditSessionId());
		
		int i = 0;
		String newPage2Name = page2 + (i++);
		
		while(wikiEngine.pageExists(newPage2Name)) {
			newPage2Name = page2 + (i++);
		}
		
		log.info("New name for page2: "+newPage2Name);
		
		//wikiEngine.rename(page2, newPage2Name);
		
		// get page with renamed reference
		WikiPage afterRename = wikiEngine.getByName(page1, wikiContext);
		
		String pageContentRenamed = afterRename.getContent();
		
		log.info("page1 content: "+pageContentRenamed);
		
		int link = -1;
		
		// look if there is any link
		if ((link = pageContentRenamed.lastIndexOf(hostURL+"/wiki/")) == -1) {
			fail("There is no link in afterRename page.");
		}
		else {
			// find link in page
			String pageName = pageContentRenamed.substring(link + (hostURL+"/wiki/").length(), pageContentRenamed.indexOf("\"", link + (hostURL+"/wiki/").length()));
			log.info("PAGENAME: "+pageName);
			
			
			// check if the link is the one renamed in dictionary
			//TODO NOT YET !
			//assertEquals(pageName, newPage2Name);
			
		}
	}
	
	public void testTranslating () {
		WikiContext wikiContext = new WikiContext(plainUser, wikiEngine.getWikiType("edit"), testWikiSession);
		
		WikiPage sourceMain = wikiEngine.getByName("Main", null);
		
		WikiPage main = wikiEngine.getByName("Main", wikiContext);
		
		log.info("Main source:\n"+sourceMain.getContent());

		ReTranslateUids reTranslateUids = new ReTranslateUids(wikiEngine);
		
		String reTranslated = reTranslateUids.getNewContent(main.getContent());
		
		log.info("Retranslated main:\n"+reTranslated);
		
		//assertEquals(reTranslated, sourceMain.getContent());
	}
}
