/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki.test;

import java.util.Vector;

import org.jboss.wiki.WikiContext;
import org.jboss.wiki.WikiPage;
import org.jboss.wiki.exceptions.WikiException;

public class GetSavePageTest extends WikiTest {
	
	public void testSaveGetPage() throws WikiException {
		WikiContext wikiContext = new WikiContext(plainUser, wikiEngine.getWikiType("html"), testWikiSession);
		
		String testPageName = createTestPage(plainUser, "This is simple conten [link] \\ Hello !");
		
		WikiPage wikiPage2 = wikiEngine.getByName(testPageName, wikiContext);
		
		String pageContent = wikiPage2.getPageContent();
		
		log.debug(pageContent);
		
		if (wikiPage2 == null) {
			wikiEngine.deletePage(testPageName);
			
			fail("Couldn't get page: "+testPageName);
		}
		
	}
	
	/*public void testGetAllPages() {
		WikiContext wikiContext = new WikiContext("test-user", wikiEngine.getWikiType("html"), testWikiSession);
		
		Set<String> pageNames = wikiEngine.getAllPageNames();
		
		for (String pageName : pageNames) {
			log.info("getting page: "+pageName);
			
			if (wikiEngine.getByName(pageName, wikiContext) == null) {
				fail("Couldn't get page: "+pageName);
			}
		}
	}*/
	
	public void testLocking() throws WikiException{
		WikiContext wikiContext = new WikiContext(plainUser, wikiEngine.getWikiType("html"), testWikiSession);
		
		WikiPage wikiPage = wikiEngine.getByName(createTestPage(plainUser, "some page content"), null);
		WikiPage wikiPage2 = wikiEngine.getByName(createTestPage(plainUser, "some page content"), null);
		
		Vector<Integer> sessIds = new Vector<Integer>();
		
		//genrate sess ids and lock pages
		for (int i = 0; i < 100; i++) {
			sessIds.add(wikiEngine.getEditSessionId());
			wikiPage.lock(plainUser, sessIds.get(i));
		}
		
		//check what wikiEngine thinks
		assertEquals(wikiEngine.arePagesEdited(), true);
		
		//unlock
		for (int i = 0; i < 100; i++) {
			wikiPage.unlock(plainUser, sessIds.get(i));
		}
		
		//check what wikiengine thinks
		assertEquals(wikiEngine.arePagesEdited(), false);
	}
}
