/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki.test;

import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

import javax.portlet.ActionRequest;
import javax.portlet.RenderRequest;

import org.jboss.logging.Logger;
import org.jboss.wiki.Credentials;
import org.jboss.wiki.WikiContext;
import org.jboss.wiki.WikiEngine;
import org.jboss.wiki.WikiPage;
import org.jboss.wiki.WikiSession;
import org.jboss.wiki.exceptions.WikiException;

import junit.framework.TestCase;

public class WikiTest extends TestCase {
	protected WikiEngine wikiEngine;
	protected WikiSession testWikiSession;
	protected Credentials notLogedIn = new TestCredentials(false, false);
	protected Credentials adminUser = new TestCredentials(true, true);
	protected Credentials plainUser = new TestCredentials(true, false);
	protected Set<String> pages;
	
	protected String TEST_PAGE_BASE_NAME = "TestPage";
	protected String hostURL = "http://HostURL.pl";
	protected String actionURL = "/Szimano";
	
	protected Logger log;
	
	public WikiTest () {
		log = Logger.getLogger(WikiTest.class);

		wikiEngine = new WikiEngine();
		testWikiSession = new TestWikiSession();
		pages = new HashSet<String>();
		
		// set session atributes
		testWikiSession.setAttribute("credentials", plainUser);
		testWikiSession.setAttribute("actionURL", actionURL);
		testWikiSession.setAttribute("hostURL", hostURL);
	}

	protected String createTestPage(Credentials user, String pageContent) throws WikiException{
		WikiContext wikiContext = new WikiContext(user, wikiEngine.getWikiType("html"), testWikiSession);
		
		if (wikiContext == null) {
			fail("There is no html wikitype");
		}
		
		
		String testPageName = TEST_PAGE_BASE_NAME;
		int i = 0;
		
		while (wikiEngine.pageExists(testPageName)) {
			testPageName = TEST_PAGE_BASE_NAME + (i++);
		}
		
		WikiPage wikiPage = new WikiPage(testPageName, user, pageContent, 1, 1, new Date(), wikiEngine, true, true);
		
		if (!wikiPage.save(wikiEngine.getEditSessionId())) {
			fail("Couldn't save page");
		}
		
		pages.add(testPageName);
		
		return testPageName;
	}
	
	protected void deleteAllTestPages() {
		boolean ret = true;
		
		for (String pageName : pages) {
			if (!wikiEngine.deletePage(pageName)) {
				ret = false;
				log.error("Couldn't delete "+pageName);
			}
		}
		
		if (!ret) {
			fail("Problems with deleting pages");
		}
	}
	
	public void testNoTests() {
		// do nothing - it's just an abstract class for other tests
	}

	@Override
	protected void tearDown() throws Exception {
		// at the end delete created testpages
		
		deleteAllTestPages();
		
		super.tearDown();
	}
}

class TestCredentials extends Credentials {
	private boolean logedIn = false;
	private boolean admin = false;
	
	public TestCredentials(boolean logedIn, boolean admin) {
		this.admin = admin;
		this.logedIn = logedIn;
	}
	
	@Override
	public String getName() {
		return (isLoggedIn()) ? ((isAdmin()) ? "admin" : "test-user") : "Unknown";
	}

	@Override
	public boolean isLoggedIn() {
		return logedIn;
	}

	@Override
	public boolean isAdmin() {
		return admin;
	}

	@Override
	public void setRenderRequest(RenderRequest rReq) {
		// Do nothing
		
	}

	@Override
	public void setActionRequest(ActionRequest rReq) {
		// Do nothing
		
	}
	
}

class TestWikiSession implements WikiSession {
	
	HashMap<String, Object> attributes;
	
	public TestWikiSession() {
		attributes = new HashMap<String, Object>();
	}

	public Object getAttribute(String attributeName) {
		return attributes.get(attributeName);
	}

	public void setAttribute(String attributeName, Object attribute) {
		attributes.put(attributeName, attribute);
	}

	public void removeAttribute(String attributeName) {
		attributes.remove(attributeName);
	}
	
}